from bx_py_utils.test_utils.snapshot import assert_snapshot

from pysmartmeter.data_classes import ObisValue
from pysmartmeter.parser import parse_obis_value
from pysmartmeter.tests import BaseTestCase
from pysmartmeter.tests.data import (
    RawObisDataMock,
    get_obis_data_block,
    test_data2obis_parser_result,
)
from pysmartmeter.utilities.serializer import serialize_values


class ParserTestCase(BaseTestCase):
    def test_raw_data_mock(self):
        rdm = RawObisDataMock()
        self.assert_verbose(
            got=tuple(rdm.iter_block(count=2)),
            excepted=(
                b'/EBZ5DD3BZ06ETA_107\r\n',
                b'\r\n',
                b'1-0:0.0.0*255(1EBZ0100000123)\r\n',
                b'1-0:96.1.0*255(1EBZ0100000123)\r\n',
                b'1-0:1.8.0*255(0010001*kWh)\r\n',
                b'1-0:96.5.0*255(001C0104)\r\n',
                b'0-0:96.8.0*255(000BEEF01)\r\n',
                b'!\r\n',
                b'/EBZ5DD3BZ06ETA_107\r\n',
                b'\r\n',
                b'1-0:0.0.0*255(1EBZ0100000123)\r\n',
                b'1-0:96.1.0*255(1EBZ0100000123)\r\n',
                b'1-0:1.8.0*255(0010002*kWh)\r\n',
                b'1-0:96.5.0*255(001C0104)\r\n',
                b'0-0:96.8.0*255(000BEEF02)\r\n',
                b'!\r\n',
            ),
        )

    def test_parse_obis_values(self):
        result = parse_obis_value('1-0:0.0.0*255(1EBZ0100000123)\r\n')
        self.assertEqual(
            result,
            ObisValue(
                key='1-0:0.0.0*255',
                key_slug='1_0_0_0_0_255',
                name='Eigentumsnummer',
                raw_value='1EBZ0100000123',
                value='1EBZ0100000123',
                raw_unit=None,
                unit=None,
            ),
        )

        result = parse_obis_value('1-0:1.8.0*255(012345*kWh)\r\n')
        self.assertEqual(
            result,
            ObisValue(
                key='1-0:1.8.0*255',
                key_slug='1_0_1_8_0_255',
                name='Zählerstand (Tariflos)',
                raw_value='012345*kWh',
                value=12345.0,
                raw_unit='kWh',
                unit='kWh',
            ),
        )

        result = parse_obis_value('0-0:96.8.0*255(00BEEF002)\r\n')
        self.assertEqual(
            result,
            ObisValue(
                key='0-0:96.8.0*255',
                key_slug='0_0_96_8_0_255',
                name='Betriebsdauer',
                raw_value='00BEEF002',
                value=200208386,
                raw_unit=None,
                unit='sec.',
            ),
        )

    def test_happy_path(self):
        parsed_bock_data = get_obis_data_block(count=2)
        self.assertEqual(len(parsed_bock_data), 2)
        self.assert_verbose(
            got=parsed_bock_data[0]['obis_values'][0],
            excepted=ObisValue(
                key='identifier',
                key_slug='identifier',
                name='Gerätetyp mit Software Version',
                raw_value='/EBZ5DD3BZ06ETA_107',
                value='EBZ5DD3BZ06ETA_107',
                raw_unit=None,
                unit=None,
            ),
        )
        self.assert_verbose(
            got=parsed_bock_data[0]['obis_values'][1],
            excepted=ObisValue(
                key='1-0:0.0.0*255',
                key_slug='1_0_0_0_0_255',
                name='Eigentumsnummer',
                raw_value='1EBZ0100000123',
                value='1EBZ0100000123',
                raw_unit=None,
                unit=None,
            ),
        )
        assert_snapshot(got=serialize_values(parsed_bock_data))

    def test_start_in_the_middle(self):
        rdm = RawObisDataMock()
        lines = [
            b'55(001C0104)\r\n',
            b'0-0:96.8.0*255(00017A9F)\r\n',
            b'!\r\n',
        ]
        lines += list(rdm.iter_block(count=1))
        parsed_bock_data = test_data2obis_parser_result(
            lines=lines,
            # verbose=True
        )
        self.assertEqual(len(parsed_bock_data), 1)
        self.assert_verbose(
            got=parsed_bock_data[0]['obis_values'][0],
            excepted=ObisValue(
                key='identifier',
                key_slug='identifier',
                name='Gerätetyp mit Software Version',
                raw_value='/EBZ5DD3BZ06ETA_107',
                value='EBZ5DD3BZ06ETA_107',
                raw_unit=None,
                unit=None,
            ),
        )
        self.assert_verbose(
            got=parsed_bock_data[0]['obis_values'][1],
            excepted=ObisValue(
                key='1-0:0.0.0*255',
                key_slug='1_0_0_0_0_255',
                name='Eigentumsnummer',
                raw_value='1EBZ0100000123',
                value='1EBZ0100000123',
                raw_unit=None,
                unit=None,
            ),
        )
        assert_snapshot(got=serialize_values(parsed_bock_data))

    def test_interim_incorrect_data(self):
        rdm = RawObisDataMock()

        # Start with good data:
        lines = list(rdm.iter_block(count=2))
        lines += [
            b'/EBZ5DD3BZ06ETA_107\r\n',
            b'\r\n',
            b'1-0:0.0.0*255(1EBZ0100000123)\r\n',
            # incorrect data:
            b'foo\rbar\n',
            b'bam\x00boom\n\r',
        ]
        # Correct data:
        lines += list(rdm.iter_block(count=1))
        parsed_bock_data = test_data2obis_parser_result(
            lines=lines,
            # verbose=True
        )
        self.assertEqual(len(parsed_bock_data), 2)
        self.assert_verbose(
            got=parsed_bock_data[0]['obis_values'][0],
            excepted=ObisValue(
                key='identifier',
                key_slug='identifier',
                name='Gerätetyp mit Software Version',
                raw_value='/EBZ5DD3BZ06ETA_107',
                value='EBZ5DD3BZ06ETA_107',
                raw_unit=None,
                unit=None,
            ),
        )
        self.assert_verbose(
            got=parsed_bock_data[0]['obis_values'][1],
            excepted=ObisValue(
                key='1-0:0.0.0*255',
                key_slug='1_0_0_0_0_255',
                name='Eigentumsnummer',
                raw_value='1EBZ0100000123',
                value='1EBZ0100000123',
                raw_unit=None,
                unit=None,
            ),
        )
        assert_snapshot(got=serialize_values(parsed_bock_data))



import dataclasses
import json
from typing import Optional, Union

from bx_py_utils.anonymize import anonymize


@dataclasses.dataclass
class MqttSettings:
    host: str
    port: int
    user_name: str
    password: str

    def as_json(self):
        data = dataclasses.asdict(self)
        data_str = json.dumps(data, ensure_ascii=False, indent=2, sort_keys=True)
        return data_str

    @classmethod
    def from_json(cls, data_str):
        data = json.loads(data_str)
        return cls(**data)

    def anonymized(self):
        data = dataclasses.asdict(self)
        if self.password:
            data['password'] = anonymize(self.password)
        return data


@dataclasses.dataclass
class ObisValue:
    """
    A "raw" Obis value
    """

    key: str  # origin name e.g.: '1-0:1.8.0*255'
    key_slug: str
    name: str  # Human readable name, e.g.: 'Zählerstand (Tariflos)'
    raw_value: str  # e.g.: '0010002*kWh'
    value: Union[str, float]  # e.g.: 10002.0
    raw_unit: Optional[str] = None  # e.g.: 'kWh'
    unit: Optional[str] = None  # e.g.: 'kWh'

    def copy(self) -> 'ObisValue':
        data = dataclasses.asdict(self)
        return ObisValue(**data)


@dataclasses.dataclass
class HomeassistantValue:
    """
    Obis value prepared for Homeassistant submission
    """

    unique_id: str
    value_key: str
    state_class: str
    obis_value: ObisValue

    @property
    def key(self):
        return self.obis_value.key


@dataclasses.dataclass
class MqttPayload:
    topic: str
    data: dict



import dataclasses


def serialize_values(data):
    if isinstance(data, list):
        return [serialize_values(entry) for entry in data]
    elif isinstance(data, dict):
        return {serialize_values(key): serialize_values(value) for key, value in data.items()}
    elif isinstance(data, (str, int, float)):
        return data
    elif dataclasses.is_dataclass(data):
        return dataclasses.asdict(data)

    raise NotImplementedError(f'Unsupported: {data!r} (type: {type(data).__name__})')



"""pysmartmeter - Collect data from Hichi Smartmeter and expose it via MQTT"""

from importlib.metadata import version


__version__ = version('pysmartmeter')
__author__ = 'Jens Diemer <pysmartmeter@jensdiemer.de>'



from unittest import TestCase

from rich.pretty import pprint


class BaseTestCase(TestCase):
    def assert_verbose(self, got, excepted):
        try:
            self.assertEqual(
                got,
                excepted,
            )
        except AssertionError:
            print('-' * 100)
            pprint(got, indent_guides=False)
            print('-' * 100)
            raise



from pathlib import Path
from unittest import TestCase

import tomli
from bx_py_utils.path import assert_is_file

import pysmartmeter
from pysmartmeter import __version__
from pysmartmeter.cli import check_code_style, fix_code_style, mypy


PACKAGE_ROOT = Path(pysmartmeter.__file__).parent.parent


class ProjectSetupTestCase(TestCase):
    def test_version(self):
        pyproject_toml_path = Path(PACKAGE_ROOT, 'pyproject.toml')
        assert_is_file(pyproject_toml_path)

        self.assertIsNotNone(__version__)

        pyproject_toml = tomli.loads(pyproject_toml_path.read_text(encoding='UTF-8'))
        pyproject_version = pyproject_toml['tool']['poetry']['version']

        self.assertEqual(__version__, pyproject_version)

    def test_code_style(self):
        fix_code_style()
        check_code_style(verbose=False)

    def test_mypy(self):
        mypy(verbose=False)



import json
import logging
import socket
import sys

import paho.mqtt.client as mqtt
from bx_py_utils.anonymize import anonymize
from rich.pretty import pprint

import pysmartmeter
from pysmartmeter.data_classes import MqttPayload, MqttSettings, ObisValue
from pysmartmeter.detect_serial import get_serial
from pysmartmeter.homeassistant import data2config, data2state, ha_convert_obis_values
from pysmartmeter.parser import ObisParser


logger = logging.getLogger(__name__)


class MqttPublisher:
    def __init__(self, settings: MqttSettings, verbose: bool = True):
        self.verbose = verbose

        client_id = self.get_client_id()

        print(f'Connect {settings.host}:{settings.port} as {client_id!r}...', end=' ')
        self.mqttc = mqtt.Client(client_id=client_id)
        self.mqttc.enable_logger(logger=logger)

        if settings.user_name and settings.password:
            print(
                f'login with user: {anonymize(settings.user_name)}'
                f' password:{anonymize(settings.password)}...',
                end=' ',
            )
            self.mqttc.username_pw_set(settings.user_name, settings.password)
        self.mqttc.connect(settings.host, port=settings.port)
        print('OK')

        self.mqttc.loop_start()

    def get_client_id(self):
        hostname = socket.gethostname()
        client_id = f'PySmartMeter v{pysmartmeter.__version__} on {hostname}'
        return client_id

    def publish(self, *, mqtt_payload: MqttPayload) -> None:
        if self.verbose:
            print('_' * 100)
            print(f'Publish MQTT topic: {mqtt_payload.topic}')
            pprint(mqtt_payload.data)

        assert self.mqttc.is_connected()
        info = self.mqttc.publish(topic=mqtt_payload.topic, payload=json.dumps(mqtt_payload.data))

        if self.verbose:
            print('publish result:', info)


def obis_values2mqtt_state(obis_values: list[ObisValue], verbose: bool = False) -> MqttPayload:
    ha_values = ha_convert_obis_values(obis_values=obis_values)
    if verbose:
        pprint(ha_values)
    return data2state(ha_values=ha_values)


def obis_values2mqtt_config(obis_values: list[ObisValue]) -> list[MqttPayload]:
    ha_values = ha_convert_obis_values(obis_values=obis_values)
    return data2config(ha_values=ha_values)


class HomeAssistantMqtt:
    def __init__(self, mqtt_publisher: MqttPublisher, verbose: bool = True):
        self.mqtt_publisher = mqtt_publisher
        self.verbose = verbose

        self.send_count = 0

    def __call__(self, *, obis_values: list[ObisValue]):
        if self.send_count % 12 == 0:
            for payload in obis_values2mqtt_config(obis_values=obis_values):
                self.mqtt_publisher.publish(mqtt_payload=payload)

        if self.verbose:
            print(f'publish values (send count: {self.send_count}):')
        self.mqtt_publisher.publish(
            mqtt_payload=obis_values2mqtt_state(obis_values=obis_values, verbose=self.verbose)
        )
        self.send_count += 1


def publish_forever(*, settings: MqttSettings, verbose):
    serial = get_serial()
    if not serial:
        print('Serial not found')
        sys.exit(1)

    mqtt_publisher = MqttPublisher(settings=settings, verbose=verbose)
    publisher = HomeAssistantMqtt(mqtt_publisher=mqtt_publisher, verbose=verbose)
    parser = ObisParser(publish_callback=publisher, verbose=verbose)

    while True:
        data = serial.readline()
        parser.feed_line(data)



import inspect
import tempfile
from pathlib import Path
from unittest import TestCase
from unittest.mock import patch

from pysmartmeter.data_classes import MqttSettings
from pysmartmeter.utilities import credentials
from pysmartmeter.utilities.credentials import get_mqtt_settings, store_mqtt_settings


class CredentialsTestCase(TestCase):
    def test_basic(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            mocked_temp_path = Path(temp_dir) / '.pysmartmeter'

            with patch.object(credentials, 'CREDENTIAL_FILE_PATH', mocked_temp_path):
                with self.assertRaises(FileNotFoundError) as exc:
                    get_mqtt_settings()
                err_msg = str(exc.exception)
                self.assertIn('No such file or directory:', err_msg)
                self.assertIn(str(mocked_temp_path), err_msg)

                path = store_mqtt_settings(
                    settings=MqttSettings(
                        host='foo',
                        port=123,
                        user_name='',
                        password='',
                    )
                )
                self.assertEqual(path, mocked_temp_path)
                self.assertEqual(
                    mocked_temp_path.read_text(),
                    inspect.cleandoc(
                        '''
                        {
                          "host": "foo",
                          "password": "",
                          "port": 123,
                          "user_name": ""
                        }
                        '''
                    ),
                )

                settings = get_mqtt_settings()
                self.assertEqual(
                    settings,
                    MqttSettings(
                        host='foo',
                        port=123,
                        user_name='',
                        password='',
                    ),
                )



import dataclasses
import re

from bx_py_utils.test_utils.snapshot import assert_snapshot

from pysmartmeter.data_classes import HomeassistantValue, MqttPayload, ObisValue
from pysmartmeter.homeassistant import (
    data2config,
    data2state,
    get_value_by_key,
    ha_convert_obis_values,
)
from pysmartmeter.publish_loop import obis_values2mqtt_config, obis_values2mqtt_state
from pysmartmeter.tests import BaseTestCase
from pysmartmeter.tests.data import get_obis_values
from pysmartmeter.utilities.serializer import serialize_values


# origin from:
# https://github.com/home-assistant/core/blob/dev/homeassistant/components/mqtt/discovery.py
# prefixed with "homeassistant" here:
TOPIC_MATCHER = re.compile(
    r'homeassistant/(?P<component>\w+)/(?:(?P<node_id>[a-zA-Z0-9_-]+)/)'
    r'?(?P<object_id>[a-zA-Z0-9_-]+)/config'
)


class HomeassistantTestCase(BaseTestCase):
    def test_ha_convert_obis_values(self):
        ha_values = ha_convert_obis_values(obis_values=get_obis_values())

        # Test examples:
        value_10180255 = get_value_by_key(values=ha_values, key='1-0:1.8.0*255')
        self.assert_verbose(
            got=value_10180255,
            excepted=HomeassistantValue(
                unique_id='ebz5dd3bz06eta107_1_0_1_8_0_255',
                value_key='value_1_0_1_8_0_255',
                state_class='total',
                obis_value=ObisValue(
                    key='1-0:1.8.0*255',
                    key_slug='1_0_1_8_0_255',
                    name='Zählerstand (Tariflos)',
                    raw_value='0010002*kWh',
                    value=10002.0,
                    raw_unit='kWh',
                    unit='kWh',
                ),
            ),
        )
        value_009680255 = get_value_by_key(values=ha_values, key='0-0:96.8.0*255')
        self.assert_verbose(
            got=value_009680255,
            excepted=HomeassistantValue(
                unique_id='ebz5dd3bz06eta107_0_0_96_8_0_255',
                value_key='value_0_0_96_8_0_255',
                state_class='total',
                obis_value=ObisValue(
                    key='0-0:96.8.0*255',
                    key_slug='0_0_96_8_0_255',
                    name='Betriebsdauer',
                    raw_value='000BEEF02',
                    value=12513026,
                    raw_unit=None,
                    unit='sec.',
                ),
            ),
        )

        assert_snapshot(got=serialize_values(ha_values))

    def test_data2config(self):
        ha_values = ha_convert_obis_values(obis_values=get_obis_values())
        mqtt_payloads = data2config(ha_values=ha_values)
        self.assertIsInstance(mqtt_payloads, list)

        payloads = []
        unique_ids = []
        for mqtt_payload in mqtt_payloads:
            self.assertIsInstance(mqtt_payload, MqttPayload)
            self.assertRegex(mqtt_payload.topic, TOPIC_MATCHER)

            # Check object_id uniqueness:
            unique_id = mqtt_payload.data['unique_id']
            self.assertNotIn(unique_id, unique_ids)
            unique_ids.append(unique_id)

            payloads.append(dataclasses.asdict(mqtt_payload))

        self.assertEqual(
            unique_ids,
            [
                'ebz5dd3bz06eta107_identifier',
                'ebz5dd3bz06eta107_1_0_0_0_0_255',
                'ebz5dd3bz06eta107_1_0_96_1_0_255',
                'ebz5dd3bz06eta107_1_0_1_8_0_255',
                'ebz5dd3bz06eta107_1_0_96_5_0_255',
                'ebz5dd3bz06eta107_0_0_96_8_0_255',
                'ebz5dd3bz06eta107_human_op_time',
            ],
        )
        assert_snapshot(got=payloads)

    def test_data2state(self):
        ha_values = ha_convert_obis_values(obis_values=get_obis_values())
        mqtt_payload = data2state(ha_values=ha_values)
        self.assertIsInstance(mqtt_payload, MqttPayload)
        self.assert_verbose(
            got=dataclasses.asdict(mqtt_payload),
            excepted={
                'topic': 'homeassistant/sensor/EBZ5DD3BZ06ETA107/state',
                'data': {
                    'value_identifier': 'EBZ5DD3BZ06ETA_107',
                    'value_1_0_0_0_0_255': '1EBZ0100000123',
                    'value_1_0_96_1_0_255': '1EBZ0100000123',
                    'value_1_0_1_8_0_255': 10002.0,
                    'value_1_0_96_5_0_255': '001C0104',
                    'value_0_0_96_8_0_255': 12513026,
                    'value_human_op_time': 4.8,
                },
            },
        )

    def test_homeassistant_mqtt_payloads(self):
        """
        Snapshot a complete configs/data MQTT payload for Home Assistant
        """
        obis_values = get_obis_values()
        payloads = obis_values2mqtt_config(obis_values=obis_values)
        payloads.append(obis_values2mqtt_state(obis_values=obis_values))
        results = []
        for mqtt_payload in payloads:
            self.assertIsInstance(mqtt_payload, MqttPayload)
            results.append(dataclasses.asdict(mqtt_payload))
        assert_snapshot(got=results)



import sys

from rich.pretty import pprint

from pysmartmeter.detect_serial import get_serial
from pysmartmeter.parser import ObisParser


def print_callback(**kwargs):
    pprint(kwargs, indent_guides=False)


def serial_dump():
    """
    Dump the output of the first working serial port.
    """
    ser = get_serial()
    if not ser:
        print('Serial not found')
        sys.exit(1)

    parser = ObisParser(publish_callback=print_callback)

    while True:
        data = ser.readline()
        print(repr(data))
        parser.feed_line(data)


if __name__ == '__main__':
    try:
        serial_dump()
    except KeyboardInterrupt:
        print('\nbye, bye\n')



import enum
import re

from rich import print as rprint
from rich.pretty import pprint

from pysmartmeter.data_classes import ObisValue
from pysmartmeter.obis_map import OBIS_KEY_MAP
from pysmartmeter.utilities.string_utils import slugify


ENCODING = 'ASCII'
TERMINATOR = '!\r\n'


class State(enum.Enum):
    identifier = 1
    separator = 2
    data = 3
    terminator = 4


ALL_STATES = [state for state in State]
DATA_SPLIT_RE = re.compile(r'(.+?)\((.+?)\)')


def parse_obis_value(line) -> ObisValue:
    matches = DATA_SPLIT_RE.findall(line)
    if not len(matches) == 1:
        raise ValueError(f'Regex. does not match for line: {line!r}')

    key, raw_value = matches[0]
    value = raw_value
    raw_unit = None
    unit = None
    if key == '0-0:96.8.0*255':  # time of operation in hex
        try:
            value = int(raw_value, 16)
        except Exception as err:
            print(f'Convert {raw_value!r} error: {err}')
        else:
            unit = 'sec.'
    elif '*' in raw_value:
        value, raw_unit = raw_value.split('*')
        try:
            value = float(value)
        except Exception as err:
            print(f'Convert {raw_value!r} error: {err}')

        unit = raw_unit

    name = OBIS_KEY_MAP.get(key, key)

    return ObisValue(
        key=key,
        key_slug=slugify(key, sep='_'),
        name=name,
        raw_value=raw_value,
        value=value,
        raw_unit=raw_unit,
        unit=unit,
    )


class ObisParser:
    def __init__(self, *, publish_callback, verbose=False):
        self.publish_callback = publish_callback
        self.verbose = verbose

        self.state = None
        self._seen_keys = None
        self._buffer = None

    def set_new_state(self, new_state):
        if self.verbose:
            print(f'state: {self.state} -> {new_state}')

        if self.state is None:
            assert (
                new_state == State.identifier
            ), f'New state {new_state.name!r} is not {State.identifier.name!r}'
        elif new_state == State.identifier:
            assert (
                self.state == State.terminator
            ), f'Current state {self.state.name!r} is not {State.terminator.name!r}'
        else:
            current_position = ALL_STATES.index(self.state)
            correct_new_state = ALL_STATES[current_position + 1]
            assert (
                new_state == correct_new_state
            ), f'New state {new_state.name!r} is not {correct_new_state.name!r}'

        self.state = new_state

    def feed_line(self, data):
        line = data.decode(ENCODING)
        if self.verbose:
            rprint(f'[code]{data!r} -> {line!r}')
        if self.state is None:
            if self.verbose:
                print('-' * 100)
                print('Start -> wait for terminator')
            if line == TERMINATOR:
                self.set_new_state(State.identifier)
        elif self.state == State.identifier:
            if self.verbose:
                print('New buffer...', end=' ')
            key = State.identifier.name
            self._buffer = [
                ObisValue(
                    key=key,
                    key_slug=key,
                    name=OBIS_KEY_MAP.get(key, key),
                    raw_value=line.strip(),
                    value=line.strip('/ \r\n'),
                )
            ]
            self._seen_keys = {key}
            if self.verbose:
                print(self._buffer)
            self.set_new_state(State.separator)
        elif self.state == State.separator:
            self.set_new_state(State.data)
        elif self.state == State.data:
            if line == TERMINATOR:
                self.set_new_state(State.terminator)
                if self.verbose:
                    print(f'Expose buffer to callback {self.publish_callback}:')
                    pprint(self._buffer)
                self.publish_callback(obis_values=self._buffer)
                self.set_new_state(State.identifier)
                if self.verbose:
                    print('-' * 100)
            else:
                try:
                    obis_value: ObisValue = parse_obis_value(line)
                    assert obis_value.key not in self._seen_keys, f'Double {obis_value.key} found!'
                except Exception as err:
                    print(f'ERROR parse line {line!r}: {err}')
                else:
                    if self.verbose:
                        print(f'Store: {obis_value}')
                    self._buffer.append(obis_value)
                    self._seen_keys.add(obis_value.key)
        else:
            raise NotImplementedError



# import doctest
# from pathlib import Path
#
# import pysmartmeter
#
#
# def load_tests(loader, tests, ignore):
#     base_path = Path(pysmartmeter.__file__).parent
#     files = [str(p.relative_to(base_path)) for p in base_path.rglob('*.py')]
#     doc_test_suite = doctest.DocFileSuite(*files, package=pysmartmeter)
#     tests.addTests(doc_test_suite)
#     return tests



from pysmartmeter.data_classes import ObisValue
from pysmartmeter.parser import ObisParser


RAW_TEST_DATA_SMALL = (
    '/EBZ5DD3BZ06ETA_107\r\n'  # Manufacturer and software version
    '\r\n'
    '1-0:0.0.0*255(1EBZ0100000123)\r\n'  # Eigentumsnummer
    '1-0:96.1.0*255(1EBZ0100000123)\r\n'  # Geräte-Identifikationsnummer
    '1-0:1.8.0*255({kwh}*kWh)\r\n'  # Summe der Momentan-Leistungen in allen Phase
    '1-0:96.5.0*255(001C0104)\r\n'  # Betriebszustand
    '0-0:96.8.0*255({epoch})\r\n'  # Betriebsdauer in Sekunden
    '!\r\n'
)
RAW_TEST_DATA_BIG = (
    '/EBZ5DD3BZ06ETA_107\r\n'  # Manufacturer and software version
    '\r\n'
    '1-0:0.0.0*255(1EBZ0100000123)\r\n'  # Eigentumsnummer
    '1-0:96.1.0*255(1EBZ0100000123)\r\n'  # Geräte-Identifikationsnummer
    '1-0:1.8.0*255(005087.48789958*kWh)\r\n'  # Zählerstand (Tariflos)
    '1-0:16.7.0*255(000236.12*W)\r\n'  # Summe der Momentan-Leistungen in allen Phase
    '1-0:36.7.0*255(000212.65*W)\r\n'  # Momentane Leistung in Phase L1
    '1-0:56.7.0*255(000000.00*W)\r\n'  # Momentane Leistung in Phase L2
    '1-0:76.7.0*255(000023.47*W)\r\n'  # Momentane Leistung in Phase L3
    '1-0:32.7.0*255(234.2*V)\r\n'  # Spannung in Phase L1
    '1-0:52.7.0*255(235.1*V)\r\n'  # Spannung in Phase L2
    '1-0:72.7.0*255(231.8*V)\r\n'  # Spannung in Phase L3
    '1-0:96.5.0*255(001C0104)\r\n'  # Betriebszustand
    '0-0:96.8.0*255(00BEEF00)\r\n'  # Betriebsdauer in Sekunden
    '!\r\n'
)


class RawObisDataMock:
    def __init__(self):
        self.kwh = 10000
        self.epoch = 0x00BEEF00

    def iter_block(self, count=1):
        for _ in range(count):
            self.kwh += 1
            self.epoch += 1
            data_str = RAW_TEST_DATA_SMALL.format(
                kwh=f'{self.kwh:07}',
                epoch=f'{self.epoch:09X}',
            )
            for line in data_str.splitlines(keepends=True):
                yield line.encode('ASCII')


def test_data2obis_parser_result(lines, verbose=False):
    class PublisherMock:
        def __init__(self):
            self.data = []

        def __call__(self, **kwargs):
            self.data.append(kwargs)

    publish_mock = PublisherMock()
    parser = ObisParser(publish_callback=publish_mock, verbose=verbose)

    for line in lines:
        parser.feed_line(line)

    return publish_mock.data


def get_obis_data_block(count=1):
    rdm = RawObisDataMock()
    parsed_bock_data = test_data2obis_parser_result(
        lines=rdm.iter_block(count=count + 1),
        # verbose=True
    )
    return parsed_bock_data


def get_obis_values() -> list[ObisValue]:
    parsed_bock_data = get_obis_data_block(count=1)
    return parsed_bock_data[0]['obis_values']



import getpass
import os
import shutil
import sys
from pathlib import Path

import typer
from bx_py_utils.path import assert_is_dir, assert_is_file
from manageprojects.utilities.publish import publish_package
from manageprojects.utilities.version_info import print_version
from rich.pretty import pprint

import pysmartmeter
from pysmartmeter.data_classes import MqttSettings
from pysmartmeter.detect_serial import print_detect_serial
from pysmartmeter.dump import serial_dump
from pysmartmeter.log_utils import log_config
from pysmartmeter.publish_loop import publish_forever
from pysmartmeter.utilities import subprocess_utils, systemd
from pysmartmeter.utilities.credentials import get_mqtt_settings, store_mqtt_settings


PACKAGE_ROOT = Path(pysmartmeter.__file__).parent.parent
assert_is_dir(PACKAGE_ROOT)
assert_is_file(PACKAGE_ROOT / 'pyproject.toml')

app = typer.Typer()


def which(file_name: str) -> Path:
    venv_bin_path = Path(sys.executable).parent
    assert venv_bin_path.is_dir()
    bin_path = venv_bin_path / file_name
    if not bin_path.is_file():
        raise FileNotFoundError(f'File {file_name}!r not found in {venv_bin_path}')
    return bin_path


def verbose_check_call(file_name, *args, **kwargs):
    file_name = which(file_name)
    kwargs.setdefault('cwd', PACKAGE_ROOT)
    subprocess_utils.verbose_check_call(
        file_name,
        *args,
        verbose=True,
        exit_on_error=True,
        **kwargs,
    )


@app.command()
def detect_serial():
    """
    Just print the detected serial port instance
    """
    log_config()
    print_detect_serial()


@app.command()
def dump():
    """
    Just dump serial output
    """
    log_config()
    serial_dump()


@app.command()
def publish_loop(log: bool = True, verbose: bool = True):
    """
    Publish current data via MQTT (endless loop)
    """
    if log:
        log_config()
    settings: MqttSettings = get_mqtt_settings()
    pprint(settings.anonymized())
    try:
        publish_forever(settings=settings, verbose=verbose)
    except KeyboardInterrupt:
        print('Bye, bye')


@app.command()
def store_settings():
    """
    Store MQTT server settings.
    """
    log_config()

    try:
        settings: MqttSettings = get_mqtt_settings()
    except FileNotFoundError:
        print('No settings stored, yet. ok.')
        print()
        print('Input settings:')
    else:
        print('Current settings:')
        pprint(settings.anonymized())
        print()
        print('Input new settings:')

    host = input('host (e.g.: "test.mosquitto.org"): ')
    if not host:
        print('Host is needed! Abort.')
        sys.exit(1)

    port = input('port (default: 1883): ')
    if port:
        port = int(port)
    else:
        port = 1883
    user_name = input('user name: ')
    password = getpass.getpass('password: ')

    settings = MqttSettings(host=host, port=port, user_name=user_name, password=password)
    file_path = store_mqtt_settings(settings)
    print(f'MQTT server settings stored into: {file_path}')


@app.command()
def debug_settings():
    """
    Display (anonymized) MQTT server username and password
    """
    log_config()
    settings: MqttSettings = get_mqtt_settings()
    pprint(settings.anonymized())


@app.command()
def debug_systemd_service():
    """
    Just print the systemd service file content
    """
    content = systemd.compile_service()
    print(content)


@app.command()
def setup_systemd_service():
    """
    Setup PySmartMeter systemd services and starts it.
    """
    systemd.write_service_file()
    systemd.enable_service()
    systemd.restart_service()


@app.command()
def systemd_status():
    """
    Call systemd status of PySmartMeter services
    """
    systemd.status()


@app.command()
def systemd_stop():
    """
    Stop PySmartMeter systemd services
    """
    systemd.stop()


def _call_darker(*, argv):
    from darker.__main__ import main as darker_main  # it's a dev requirements

    # Work-a-round for:
    #
    #   File ".../site-packages/darker/linting.py", line 148, in _check_linter_output
    #     with Popen(  # nosec
    #   ...
    #   File "/usr/lib/python3.10/subprocess.py", line 1845, in _execute_child
    #     raise child_exception_type(errno_num, err_msg, err_filename)
    # FileNotFoundError: [Errno 2] No such file or directory: 'flake8'
    #
    # Just add .venv/bin/ to PATH:
    venv_path = Path(sys.executable).parent
    assert_is_dir(venv_path)
    assert_is_file(venv_path / 'flake8')
    venv_path = str(venv_path)
    if venv_path not in os.environ['PATH']:
        os.environ['PATH'] = venv_path + os.pathsep + os.environ['PATH']

    result = darker_main(argv=argv)
    print(f'Darker returns: {result!r}')


@app.command()
def fix_code_style():
    """
    Fix code style via darker
    """
    _call_darker(argv=['--color'])


@app.command()
def check_code_style(verbose: bool = True):
    from flake8.main.cli import main as flake8_main  # it's a dev requirements

    _call_darker(argv=['--color', '--check'])
    if verbose:
        argv = ['--verbose']
    else:
        argv = []

    flake8_main(argv=argv)


@app.command()
def mypy(verbose: bool = False):
    """Run Mypy (configured in pyproject.toml)"""
    args = ['mypy']
    if verbose:
        args.append('--verbose')
    verbose_check_call(*args, PACKAGE_ROOT)


@app.command()  # Just add this command to help page
def test():
    """
    Run unittests
    """
    # Use the CLI from unittest module and pass all args to it:
    verbose_check_call(sys.executable, '-m', 'unittest', *sys.argv[2:])


@app.command()
def coverage():
    """
    Run and show coverage.
    """
    verbose_check_call('coverage', 'run')
    verbose_check_call('coverage', 'report', '--fail-under=30')
    verbose_check_call('coverage', 'json')


@app.command()
def publish():
    """
    Build and upload this project to PyPi
    """
    log_config()
    test()  # Don't publish a broken state

    # TODO: Use publish from manageprojects!

    dist_path = PACKAGE_ROOT / 'dist'
    if dist_path.exists():
        shutil.rmtree(dist_path)

    subprocess_utils.verbose_check_call('poetry', 'build', verbose=True, exit_on_error=True)

    verbose_check_call('twine', 'check', 'dist/*')
    verbose_check_call('twine', 'upload', 'dist/*')


def main():
    print_version(pysmartmeter)

    if len(sys.argv) >= 2 and sys.argv[1] == 'test':
        # Just use the CLI from unittest with all available options and origin --help output ;)
        return test()
    else:
        app()



"""
    https://github.com/home-assistant/core/blob/dev/homeassistant/components/mqtt/discovery.py
"""
from bx_py_utils.humanize.time import human_timedelta

from pysmartmeter import ha_const
from pysmartmeter.data_classes import HomeassistantValue, MqttPayload, ObisValue
from pysmartmeter.obis_map import (
    DEFAULT_HS_STATE,
    OBIS_KEY2HA_STATE_CLASS,
    OBIS_OPERATION_DURATION_KEY,
)
from pysmartmeter.utilities.string_utils import slugify


HA_COMPONENT = 'sensor'


def value_key(raw_key):
    return f'value_{slugify(raw_key)}'


def get_value_by_key(values, key: str):
    for value in values:
        if value.key == key:
            return value
    raise KeyError(key)


def data2config(ha_values: list[HomeassistantValue]) -> list[MqttPayload]:
    identifier_value = get_value_by_key(values=ha_values, key='identifier')
    identifier_raw = identifier_value.obis_value.value
    assert identifier_raw
    identifier_slug = slugify(identifier_raw)

    state_topic = f'{ha_const.DEFAULT_PREFIX}/{HA_COMPONENT}/{identifier_slug}/state'

    identifiers = [ha_value.unique_id for ha_value in ha_values]

    payloads = []
    for ha_value in ha_values:
        config_topic = f'{ha_const.DEFAULT_PREFIX}/{HA_COMPONENT}/{ha_value.unique_id}/config'
        value_template = '{{ value_json.%(value_key)s }}' % dict(value_key=ha_value.value_key)

        payloads.append(
            MqttPayload(
                topic=config_topic,
                data={
                    ha_const.CONF_DEVICE_CLASS: 'energy',
                    ha_const.CONF_DEVICE: {
                        'name': identifier_raw,
                        'identifiers': identifiers,
                    },
                    'name': ha_value.obis_value.name,
                    'unique_id': ha_value.unique_id,
                    'state_class': ha_value.state_class,
                    ha_const.CONF_STATE_TOPIC: state_topic,
                    'unit_of_measurement': ha_value.obis_value.unit,
                    'value_template': value_template,
                },
            )
        )

    return payloads


def data2state(ha_values: list[HomeassistantValue]) -> MqttPayload:
    identifier_value = get_value_by_key(values=ha_values, key='identifier')
    identifier_raw = identifier_value.obis_value.value
    assert identifier_raw
    identifier_slug = slugify(identifier_raw)

    payload = {ha_value.value_key: ha_value.obis_value.value for ha_value in ha_values}
    return MqttPayload(
        topic=f'{ha_const.DEFAULT_PREFIX}/{HA_COMPONENT}/{identifier_slug}/state',
        data=payload,
    )


def ha_convert_obis_values(*, obis_values: list[ObisValue]) -> list[HomeassistantValue]:
    identifier_value = get_value_by_key(values=obis_values, key='identifier')
    identifier_raw = identifier_value.value
    assert identifier_raw
    identifier_slug = slugify(identifier_raw).lower()

    ha_values = []
    for value in obis_values:
        state_class = OBIS_KEY2HA_STATE_CLASS.get(value.key, DEFAULT_HS_STATE)
        unique_id = f'{identifier_slug}_{value.key_slug}'
        ha_values.append(
            HomeassistantValue(
                unique_id=unique_id,
                value_key=f'value_{value.key_slug}',
                state_class=state_class,
                obis_value=value,
            )
        )

    # Get the RAW operation time in seconds:
    try:
        operation_duration = get_value_by_key(values=obis_values, key=OBIS_OPERATION_DURATION_KEY)
    except KeyError:
        pass
    else:
        # Add a human-readable operation time entry:
        op_sec = operation_duration.value
        obis_value: ObisValue = operation_duration.copy()
        op_str = human_timedelta(op_sec)
        value, unit = op_str.split('\xa0')
        obis_value.value = float(value)  # type: ignore[arg-type]
        obis_value.unit = unit
        ha_values.append(
            HomeassistantValue(
                unique_id=f'{identifier_slug}_human_op_time',
                value_key='value_human_op_time',
                state_class=DEFAULT_HS_STATE,
                obis_value=obis_value,
            )
        )

    return ha_values



import getpass
import os
import sys
from pathlib import Path

from pysmartmeter import __version__
from pysmartmeter.utilities.subprocess_utils import verbose_check_call


SERVICE_NAME = 'pysmartmeter.service'
SYSTEMD_SERVICE_PATH = Path(f'/etc/systemd/system/{SERVICE_NAME}')

SYSTEMD_SERVICE_TEMPLATE = '''
[Unit]
Description=PySmartMeter {version}
After=syslog.target network.target

[Service]
User={user}
Group={group}
WorkingDirectory={work_dir}

ExecStart={python_bin} -m pysmartmeter publish-loop --no-log --no-verbose

Restart=always
RestartSec=5s
StandardOutput=syslog
StandardError=syslog
SyslogIdentifier=PySmartMeter

[Install]
WantedBy=multi-user.target
'''


def compile_service():

    user_name = os.environ.get('SUDO_USER', getpass.getuser())
    work_dir = Path(sys.executable).parent

    content = SYSTEMD_SERVICE_TEMPLATE.format(
        version=__version__,
        user=user_name,
        group=user_name,
        work_dir=work_dir,
        python_bin=sys.executable,
    )
    return content


def write_service_file():
    content = compile_service()
    print(f'Write "{SYSTEMD_SERVICE_PATH}"...')
    try:
        SYSTEMD_SERVICE_PATH.write_text(content, encoding='UTF-8')
    except PermissionError as err:
        print(f'ERROR: {err}')
        print('Please restart this command with "sudo" ;)')
        sys.exit(1)

    verbose_check_call('systemctl', 'daemon-reload')


def call_service_command(command: str):
    verbose_check_call('systemctl', command, SERVICE_NAME)


def enable_service():
    print(f'Enable systemd service {SERVICE_NAME!r}')
    call_service_command('enable')


def restart_service():
    print(f'(re-)start systemd service {SERVICE_NAME!r}')
    call_service_command('restart')


def status():
    call_service_command('status')


def stop():
    call_service_command('stop')



# from: https://github.com/home-assistant/core/blob/dev/homeassistant/const.py
CONF_DEVICE = "device"
CONF_DEVICE_CLASS = "device_class"

# from: https://github.com/home-assistant/core/blob/dev/homeassistant/components/mqtt/const.py
CONF_STATE_TOPIC = "state_topic"
DEFAULT_PREFIX = "homeassistant"  # Discovery prefix



import shlex
import subprocess
import sys


def verbose_check_call(*popenargs, verbose: bool = True, exit_on_error: bool = False, **kwargs):
    print(f'+{shlex.join(str(part) for part in popenargs)}')
    try:
        subprocess.check_call(popenargs, **kwargs)
    except subprocess.CalledProcessError as err:
        if verbose:
            print(f'Process "{popenargs[0]}" finished with exit code {err.returncode!r}')
        if exit_on_error:
            sys.exit(err.returncode)
        raise



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pysmartmeter', 'pysmartmeter.tests', 'pysmartmeter.utilities']

package_data = \
{'': ['*']}

install_requires = \
['bx_py_utils', 'paho-mqtt', 'pyserial', 'rich', 'typer']

entry_points = \
{'console_scripts': ['pysmartmeter = pysmartmeter.main:main']}

setup_kwargs = {
    'name': 'pysmartmeter',
    'version': '0.2.1rc1',
    'description': 'Collect data from Hichi Smartmeter and expose it via MQTT',
    'long_description': '# pysmartmeter\n\nCollect data from Hichi Smartmeter aka `volkszaehler.org` (USB Version) and expose it via MQTT.\n[![tests](https://github.com/jedie/pysmartmeter/actions/workflows/tests.yml/badge.svg?branch=main)](https://github.com/jedie/pysmartmeter/actions/workflows/tests.yml)\n[![codecov](https://codecov.io/github/jedie/pysmartmeter/branch/main/graph/badge.svg)](https://codecov.io/github/jedie/pysmartmeter)\n[![pysmartmeter @ PyPi](https://img.shields.io/pypi/v/pysmartmeter?label=pysmartmeter%20%40%20PyPi)](https://pypi.org/project/pysmartmeter/)\n[![Python Versions](https://img.shields.io/pypi/pyversions/pysmartmeter)](https://github.com/jedie/pysmartmeter/blob/main/pyproject.toml)\n[![License GPL-3.0-or-later](https://img.shields.io/pypi/l/pysmartmeter)](https://github.com/jedie/pysmartmeter/blob/main/LICENSE)\n\nIn the end it can looks like the following [Home Assistant](https://www.home-assistant.io/) dashboard screenshot, using [MQTT integration](https://www.home-assistant.io/integrations/mqtt):\n\n![2022-11-21_13-47.png](https://raw.githubusercontent.com/jedie/jedie.github.io/master/screenshots/pysmartmeter/2022-11-21_13-47.png "2022-11-21_13-47.png")\n\nWith my "eBZ DD3" energy meterby eBZ GmbH the values update live every second ;)\n\n\n## quickstart\n\nInstall minimum requirements, e.g.:\n```bash\n~$ sudo apt install make python3-venv\n```\n\nClone sources and install project:\n```bash\n~$ git clone https://github.com/jedie/pysmartmeter.git\n~$ cd pysmartmeter\n~/pysmartmeter$ make install-poetry\n~/pysmartmeter$ make install\n~/pysmartmeter$ ./cli.sh --help\n+ exec .venv/bin/python -m pysmartmeter --help\nPySmartMeter v0.1.0\n\n Usage: python -m pysmartmeter [OPTIONS] COMMAND [ARGS]...\n\n╭─ Options ─────────────────────────────────────────────────────────────────────────────────────────╮\n│ --install-completion        [bash|zsh|fish|powershell|pwsh]  Install completion for the specified │\n│                                                              shell.                               │\n│                                                              [default: None]                      │\n│ --show-completion           [bash|zsh|fish|powershell|pwsh]  Show completion for the specified    │\n│                                                              shell, to copy it or customize the   │\n│                                                              installation.                        │\n│                                                              [default: None]                      │\n│ --help                                                       Show this message and exit.          │\n╰───────────────────────────────────────────────────────────────────────────────────────────────────╯\n╭─ Commands ────────────────────────────────────────────────────────────────────────────────────────╮\n│ check-code-style                                                                                  │\n│ coverage                     Run and show coverage.                                               │\n│ debug-settings               Display (anonymized) MQTT server username and password               │\n│ debug-systemd-service        Just print the systemd service file content                          │\n│ detect-serial                Just print the detected serial port instance                         │\n│ dump                         Just dump serial output                                              │\n│ fix-code-style               Fix code style via darker                                            │\n│ mypy                         Run Mypy (configured in pyproject.toml)                              │\n│ publish-loop                 Publish current data via MQTT (endless loop)                         │\n│ setup-systemd-service        Setup PySmartMeter systemd services and starts it.                   │\n│ store-settings               Store MQTT server settings.                                          │\n│ systemd-status               Call systemd status of PySmartMeter services                         │\n│ systemd-stop                 Stop PySmartMeter systemd services                                   │\n│ test                         Run unittests                                                        │\n╰───────────────────────────────────────────────────────────────────────────────────────────────────╯\n```\n\nTest if you Hichi Smartmeter with CP2102 USB to UART Bridge Controller will be found, e.g.:\n```bash\n~/pysmartmeter$ ./cli.sh detect-serial\n```\n\nMaybe you didn\'t have permissions to access the port, e.g.:\n```bash\n~/pysmartmeter$ ./cli.sh dump\n...\ntry: /dev/ttyUSB0 CP2102 USB to UART Bridge Controller - CP2102 USB to UART Bridge Controller USB VID:PID=10C4:EA60\n/dev/ttyUSB0 file mode: 0o20660\n/dev/ttyUSB0 user ID: 0\n/dev/ttyUSB0 user group ID: 20\n/dev/ttyUSB0 user group: \'dialout\'\nERROR: [Errno 13] could not open port /dev/ttyUSB0: [Errno 13] Permission denied: \'/dev/ttyUSB0\'\n...\n```\n\nFix fiy by add the user to the group, e.g.:\n```bash\nsudo usermod -a -G dialout $USER\n```\n\n## publish smartmeter data via MQTT\n\nYou have to store your MQTT settings (host, port, username, password) one time, e.g.:\n```bash\n~/pysmartmeter$ ./cli.sh store-settings\n```\nThis will create a JSON file here: `~/.pysmartmeter`\n\nYou can test reading this settings file with:\n```bash\n~/pysmartmeter$ ./cli.sh debug-settings\n```\n\nSetup systemd service:\n```bash\n~/pysmartmeter$ sudo ./cli.sh setup-systemd-service\n```\nThis will create a systemd service that automaticly starts on every boot.\n\nCheck if service is running:\n```bash\n~/pysmartmeter$ sudo ./cli.sh systemd-status\n```\n\nIf everything is fine: Go to your [Home Assistant and check the MQTT integration](https://www.home-assistant.io/integrations/mqtt/)\nThe device discovery will be made automaticly.\n\n# various links\n\n* https://github.com/pyserial/pyserial\n* https://github.com/eclipse/paho.mqtt.python\n* https://github.com/eclipse/mosquitto\n* https://dewiki.de/Lexikon/OBIS-Kennzahlen (de) | https://www.promotic.eu/en/pmdoc/Subsystems/Comm/PmDrivers/IEC62056_OBIS.htm (en)\n* https://www.photovoltaikforum.com/thread/145886-habe-lesk%C3%B6pfe-mit-usb-%C3%BCber/ (de)\n* https://www.heise.de/tests/Ausprobiert-Guenstiger-IR-Lesekopf-fuer-Smart-Meter-mit-Tastmota-Firmware-7065559.html (de)\n* https://www.home-assistant.io\n',
    'author': 'Jens Diemer',
    'author_email': 'pysmartmeter@jensdiemer.de',
    'maintainer': 'Jens Diemer',
    'maintainer_email': 'pysmartmeter@jensdiemer.de',
    'url': 'https://github.com/jedie/pysmartmeter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0.0',
}


setup(**setup_kwargs)



# https://dewiki.de/Lexikon/OBIS-Kennzahlen
OBIS_KEY_MAP = {
    'identifier': 'Gerätetyp mit Software Version',
    '1-0:0.0.0*255': 'Eigentumsnummer',
    '1-0:96.1.0*255': 'Geräte-Identifikationsnummer',
    '1-0:1.8.0*255': 'Zählerstand (Tariflos)',
    '1-0:16.7.0*255': 'Momentan-Leistungen',
    '1-0:36.7.0*255': 'Momentane Leistung L1',
    '1-0:56.7.0*255': 'Momentane Leistung L2',
    '1-0:76.7.0*255': 'Momentane Leistung L3',
    '1-0:32.7.0*255': 'Spannung L1',
    '1-0:52.7.0*255': 'Spannung L2',
    '1-0:72.7.0*255': 'Spannung L3',
    '1-0:96.5.0*255': 'Betriebszustand',
    '0-0:96.8.0*255': 'Betriebsdauer',
}

OBIS_OPERATION_DURATION_KEY = '0-0:96.8.0*255'

DEFAULT_HS_STATE = 'measurement'
OBIS_KEY2HA_STATE_CLASS = {
    '1-0:1.8.0*255': 'total',
    '0-0:96.8.0*255': 'total',
}



from pysmartmeter.tests import BaseTestCase
from pysmartmeter.utilities.string_utils import slugify


class UtilitiesTestCase(BaseTestCase):
    def test_slugify(self):
        self.assertEqual(slugify('Test äöü ß'), 'Testaou')
        self.assertEqual(slugify('1-0:96.1.0*255'), '109610255')
        self.assertEqual(slugify('EBZ5DD3BZ06ETA_107'), 'EBZ5DD3BZ06ETA107')

        self.assertEqual(slugify('Test äöü ß', sep='-'), 'Test-aou')
        self.assertEqual(slugify('1-0:96.1.0*255', sep='-'), '1-0-96-1-0-255')
        self.assertEqual(slugify('EBZ5DD3BZ06ETA_107', sep='-'), 'EBZ5DD3BZ06ETA-107')



from pathlib import Path

from pysmartmeter.data_classes import MqttSettings


CREDENTIAL_FILE_PATH = Path('~/.pysmartmeter').expanduser()


def store_mqtt_settings(settings: MqttSettings):
    print(f'Store MQTT server settings into: {CREDENTIAL_FILE_PATH}')

    data_str = settings.as_json()

    CREDENTIAL_FILE_PATH.touch(exist_ok=True)
    CREDENTIAL_FILE_PATH.chmod(0o600)
    CREDENTIAL_FILE_PATH.write_text(data_str, encoding='UTF-8')
    return CREDENTIAL_FILE_PATH


def get_mqtt_settings() -> MqttSettings:
    try:
        data_str = CREDENTIAL_FILE_PATH.read_text(encoding='UTF-8')
    except FileNotFoundError as err:
        print(f'ERROR: Error reading config: {err}')
        print('(Hint save settings first with: "./cli.sh store-settings")')
        raise FileNotFoundError(err)
    settings = MqttSettings.from_json(data_str)
    return settings



import grp
from pathlib import Path

import serial
from serial.tools.list_ports_posix import comports


def get_serial(
    baudrate=9600,
    parity=serial.PARITY_EVEN,
    stopbits=serial.STOPBITS_ONE,
    bytesize=serial.SEVENBITS,
    timeout=1,
    terminator=b'\r\n',
    verbose=True,
):
    """
    Get the first "working" serial instance back.
    """
    for port, desc, hwid in comports(include_links=False):
        if verbose:
            print('_' * 100)
            print(f'try: {port} {desc} {hwid}')

        port_stat = Path(port).stat()
        print(f'{port} file mode:', oct(port_stat.st_mode))
        print(f'{port} user ID:', port_stat.st_uid)
        print(f'{port} user group ID:', port_stat.st_gid)
        user_group_name = grp.getgrgid(port_stat.st_gid).gr_name
        print(f'{port} user group: {user_group_name!r}')

        try:
            ser = serial.Serial(
                port,
                baudrate=baudrate,
                parity=parity,
                stopbits=stopbits,
                bytesize=bytesize,
                timeout=timeout,
            )
        except Exception as err:
            if verbose:
                print(f'ERROR: {err}')
                print()
                print('Hint:')
                print(f'\tsudo usermod -a -G {user_group_name} $USER')
                print('and try again ;)')
                print('-' * 100)
            continue

        if verbose:
            print(f'Read from {ser}...', end=' ', flush=True)
        try:
            data = ser.readline()
        except Exception as err:
            if verbose:
                print(f'ERROR: {err}')
            continue
        else:
            if verbose:
                print(data)
            if data.endswith(terminator):
                return ser


def print_detect_serial():
    print('Detect Serial...')
    ser = get_serial()
    print(ser)
